<?php

namespace AustraliaPost\Extensions\Business\Endpoints;

use AustraliaPost\Extensions\Business\API\Business;
use AustraliaPost\Extensions\Business\API\Wrapper\Label;
use AustraliaPost\Extensions\Business\Helpers\Utilities;
use WC_Order;

class Generate_Labels_By_Order_Endpoint extends Abstract_Endpoint {

	public function callback( $data ) {

		if ( ! isset( $data['order_reference'] ) ) {
			wp_die( 'The order reference must be present.' );
		}

		$order_reference = sanitize_text_field( $data['order_reference'] );

		if ( ! $order_reference ) {
			wp_die( 'The order reference must be present.' );
		}
		$order = $this->get_aupost_order( $order_reference );

		if ( $order === null ) {
			wp_die( 'The order reference does not exist.' );
		}

		$order = unserialize( $order );

		$labelsRequest = $this->get_label_request( $order['shipments'] );

		$response = ( new Business() )->generate_label( $labelsRequest );

		if ( ! isset( $response['labels'] ) ) {
			return $response;
		}
		$this->store_labels_for_each_wc_order( $order['shipments'], $response );

		return [ 'url' => $response['labels'][0]['url'] ];

	}

	public function action() {
		return 'generate_labels_by_order';
	}

	private function get_label_request( $shipments ) {
		$label_request = new Label();
		$label_request->setIsBranded( Utilities::get_method_setting( 'label_branded' ) !== 'no' );
		$label_request->setLabelLayouts( Utilities::get_method_setting( 'label_layout' ) );

		foreach ( $shipments as $shipment ) {

			$label_request->addShipment( [
				'shipment_id' => $shipment['shipment_id'],
				'items'       => $shipment['items']
			] );
		}

		return $label_request;
	}

	private function get_aupost_order( $order_reference ) {
		global $wpdb;
		$key   = Create_Order_From_Shipments_Endpoint::OPTION_KEY_ORDER . $order_reference;
		$order = $wpdb->get_col( $wpdb->prepare( "SELECT option_value FROM {$wpdb->prefix}options WHERE option_name = %s ORDER BY option_id DESC", [ $key ] ) );

		if ( ! $order ) {
			return null;
		}

		return $order[0];
	}

	private function store_labels_for_each_wc_order( $shipments, $label_response )
	{
		foreach ( $shipments as $shipment ) {
			$order_id = $this->get_order_id_from_shipment_reference( $shipment['shipment_reference'] );
			if ($order_id === 0) {
				continue;
			}
			$order    = new WC_Order( $order_id );
			$order->update_meta_data( Generate_Labels_Endpoint::LABELS_LABELS_META_KEY, Utilities::_json_encode( $label_response ) );
			$order->save();
		}
	}


}
